package com.diagbot.client;

import com.diagbot.client.hystrix.CdssCoreHystrix;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.HasStaticKnowledgeVO;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.PushPlanVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: CDSS核心类客户端
 * @author: gaodm
 * @time: 2020/7/28 9:29
 */
@FeignClient(name = "cdss-core", url = "${cdss-core.url}", fallback = CdssCoreHystrix.class)
public interface CdssCoreClient {

    @PostMapping("/core/indication")
    RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO);

    @PostMapping("/core/push")
    RespDTO<PushDTO> push(@RequestBody @Valid PushVO pushVo);

    /**
     * 推送持续检验检查计划
     */
    @PostMapping("/core/pushPlan")
    RespDTO<PushPlanDTO> pushPlan(@RequestBody PushPlanVO pushPlanVO);

    /**
     * 化验大项、化验小项、辅检、诊断、药品、手术等检索
     *
     * @param retrievalVO
     * @return
     */
    @PostMapping("/retrieval/index")
    RespDTO<RetrievalDTO> index(@RequestBody @Valid RetrievalVO retrievalVO);

    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    @PostMapping("/retrieval/staticKnowledgeIndex")
    RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@RequestBody @Valid StaticKnowledgeIndexVO staticKnowledgeIndexVO);

    /**
     * 更新是否有静态知识状态
     *
     * @param hasStaticKnowledgeVO
     * @return
     */
    @PostMapping("/staticKnowledge/updateHasInfoStatus")
    RespDTO<Boolean> updateHasInfoStatus(@Valid @RequestBody HasStaticKnowledgeVO hasStaticKnowledgeVO);

    /**
     * 术语批量校验
     *
     * @param conceptVO
     * @return
     */
    @PostMapping("/retrieval/getConceptNames")
    RespDTO<List<String>> getConceptNames(@Valid @RequestBody ConceptVO conceptVO);
}


