package com.diagbot.entity;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 化验公表映射
 * </p>
 *
 * @author zhaops
 * @since 2020-07-28
 */
@Data
@TableName("tran_lis_config")
public class LisConfig implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 是否删除,N:未删除，Y:删除
     */
    private String isDeleted;

    /**
     * 记录创建时间
     */
    private Date gmtCreate;

    /**
     * 记录修改时间,如果时间是1970年则表示纪录未修改
     */
    private Date gmtModified;

    /**
     * 创建人，0表示无创建人值
     */
    private String creator;

    /**
     * 修改人,如果为0则表示纪录未修改
     */
    private String modifier;

    /**
     * 医院id
     */
    private Long hospitalId;

    /**
     * 检验套餐
     */
    @Excel(name = "检验套餐", width = 40, orderNum = "1", isImportField = "true")
    @NotBlank(message = "请输入检验套餐")
    private String hisName;

    /**
     * 检验细项
     */
    @Excel(name = "检验细项", width = 40, orderNum = "2", isImportField = "true")
    private String hisDetailName;

    /**
     * 标准检验项目
     */
    @Excel(name = "标准检验项目", width = 40, orderNum = "4", isImportField = "true")
    @NotBlank(message = "请输入标准检验项目")
    private String uniqueName;

    /**
     * 对应项目编码
     */
    //@Excel(name = "对应项目编码", width = 40, orderNum = "3")
    private String uniqueCode;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getIsDeleted() {
        return isDeleted;
    }

    public void setIsDeleted(String isDeleted) {
        this.isDeleted = isDeleted;
    }

    public Date getGmtCreate() {
        return gmtCreate;
    }

    public void setGmtCreate(Date gmtCreate) {
        this.gmtCreate = gmtCreate;
    }

    public Date getGmtModified() {
        return gmtModified;
    }

    public void setGmtModified(Date gmtModified) {
        this.gmtModified = gmtModified;
    }

    public String getCreator() {
        return creator;
    }

    public void setCreator(String creator) {
        this.creator = creator;
    }

    public String getModifier() {
        return modifier;
    }

    public void setModifier(String modifier) {
        this.modifier = modifier;
    }

    public Long getHospitalId() {
        return hospitalId;
    }

    public void setHospitalId(Long hospitalId) {
        this.hospitalId = hospitalId;
    }

    public String getHisName() {
        return hisName;
    }

    public void setHisName(String hisName) {
        this.hisName = hisName;
    }

    public String getHisDetailName() {
        return hisDetailName;
    }

    public void setHisDetailName(String hisDetailName) {
        this.hisDetailName = hisDetailName;
    }

    public String getUniqueName() {
        return uniqueName;
    }

    public void setUniqueName(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    public String getUniqueCode() {
        return uniqueCode;
    }

    public void setUniqueCode(String uniqueCode) {
        this.uniqueCode = uniqueCode;
    }

    @Override
    public String toString() {
        return "LisConfig{" +
                "id=" + id +
                ", isDeleted=" + isDeleted +
                ", gmtCreate=" + gmtCreate +
                ", gmtModified=" + gmtModified +
                ", creator=" + creator +
                ", modifier=" + modifier +
                ", hospitalId=" + hospitalId +
                ", hisName=" + hisName +
                ", hisDetailName=" + hisDetailName +
                ", uniqueName=" + uniqueName +
                ", uniqueCode=" + uniqueCode +
                "}";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;
        LisConfig lisConfig = (LisConfig) o;
        return Objects.equals(id, lisConfig.id)
                && Objects.equals(isDeleted, lisConfig.isDeleted)
                && Objects.equals(hospitalId, lisConfig.hospitalId)
                && Objects.equals(hisName, lisConfig.hisName)
                && Objects.equals(hisDetailName, lisConfig.hisDetailName)
                && Objects.equals(uniqueName, lisConfig.uniqueName)
                && Objects.equals(uniqueCode, lisConfig.uniqueCode);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, isDeleted, hospitalId, hisName, hisDetailName, uniqueName, uniqueCode);
    }
}