package com.diagbot.facade;

import com.diagbot.dto.PushDTO;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.CoreUtil;
import com.diagbot.vo.SearchData;
import io.github.lvyahui8.spring.facade.DataFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/4 14:09
 */
@Component
public class AssembleFacade {
    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;

    /**
     * 入参映射
     *
     * @param searchData
     * @return
     */
    public SearchData assembleData(SearchData searchData) {
        searchData.setAgeNum(CoreUtil.convertAge(searchData.getAge()));
        HospitalInfo hospitalInfo = hospitalInfoFacade.getById(searchData.getHospitalId());
        Integer isConnect = 0;
        if (hospitalInfo != null && hospitalInfo.getIsDeleted().equals(IsDeleteEnum.N.getKey())) {
            isConnect = hospitalInfo.getConnect();
        }
        if (isConnect.equals(1)) {
            try {
                Map<String, Object> invokeParams = new HashMap<>();
                invokeParams.put("searchData", searchData);
                searchData = DataFacade.get("assembleDataAll", invokeParams, SearchData.class);
            } catch (Exception e) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "入参映射出错" + e.getMessage());
            }
        }
        return searchData;
    }

    /**
     * 出参映射
     *
     * @param data
     * @param hospitalId
     * @return
     */
    public PushDTO assemblePushDTO(PushDTO data, Long hospitalId) {
        HospitalInfo hospitalInfo = hospitalInfoFacade.getById(hospitalId);
        Integer isConnect = 0;
        if (hospitalInfo != null && hospitalInfo.getIsDeleted().equals(IsDeleteEnum.N.getKey())) {
            isConnect = hospitalInfo.getConnect();
        }
        if (isConnect.equals(1)) {
            try {
                Map<String, Object> invokeParams = new HashMap<>();
                invokeParams.put("pushDTO", data);
                invokeParams.put("hospitalId", hospitalId);
                data = DataFacade.get("assemblePushAll", invokeParams, PushDTO.class);
            } catch (Exception e) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "出参映射出错" + e.getMessage());
            }
        }
        return data;
    }


}