package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.LisConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.LisConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.LisConfigListVO;
import com.diagbot.vo.LisConfigPageVO;
import com.diagbot.vo.RetrievalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:03
 */
@Component
public class LisConfigFacade {
    @Autowired
    private LisConfigService lisConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param lisConfig
     * @return
     */
    public Boolean isExistRecord(LisConfig lisConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", lisConfig.getHisName())
                .eq("unique_name", lisConfig.getUniqueName());
        if (StringUtil.isBlank(lisConfig.getHisDetailName())) {
            queryWrapper.and(i -> i.isNull("his_detail_name")
                    .or()
                    .eq("his_detail_name", ""));
        } else {
            queryWrapper
                    .eq("his_detail_name", lisConfig.getHisDetailName());
        }
        LisConfig oldRecord = lisConfigService.getOne(queryWrapper, false);
        if (lisConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (lisConfig.getId() != null
                && oldRecord != null
                && !lisConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param lisConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(LisConfig lisConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        lisConfig.setHospitalId(Long.valueOf(hospitalId));
        lisConfig.setModifier(userId);
        lisConfig.setGmtModified(now);
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", lisConfig.getHisName())
                .eq("unique_name", lisConfig.getUniqueName());
        if (StringUtil.isBlank(lisConfig.getHisDetailName())) {
            queryWrapper.and(i -> i.isNull("his_detail_name")
                    .or()
                    .eq("his_detail_name", ""));
        } else {
            queryWrapper
                    .eq("his_detail_name", lisConfig.getHisDetailName());
        }
        LisConfig oldRecord = lisConfigService.getOne(queryWrapper, false);
        if (lisConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (lisConfig.getId() != null
                && oldRecord != null
                && !lisConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (lisConfig.getId() == null) {
            lisConfig.setCreator(userId);
            lisConfig.setGmtCreate(now);
        }
        if (lisConfig.getIsDeleted() == null) {
            lisConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        lisConfigService.saveOrUpdate(lisConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param lisConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(LisConfigListVO lisConfigListVO) {
        if (ListUtil.isEmpty(lisConfigListVO.getLisConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(lisConfigListVO.getLisConfigList());
    }

    /**
     * 批量保存
     *
     * @param lisConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<LisConfig> lisConfigList) {
        if (ListUtil.isEmpty(lisConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤套餐或公表名为空的数据
        lisConfigList = lisConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Map<String, Long>>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        lisConfigList.forEach(lisConfig -> {
            lisConfig.setHospitalId(Long.valueOf(hospitalId));
            lisConfig.setModifier(userId);
            lisConfig.setGmtModified(now);
            if (lisConfig.getId() == null) {
                if (configMap.containsKey(lisConfig.getHisName())) {
                    if (lisConfig.getHisDetailName() == null
                            && configMap.get(lisConfig.getHisName()).containsKey("")) {
                        if (configMap.get(lisConfig.getHisName()).get("").containsKey(lisConfig.getUniqueName())) {
                            deleteIds.add(configMap.get(lisConfig.getHisName()).get("").get(lisConfig.getUniqueName()));
                        }
                    } else if (configMap.get(lisConfig.getHisName()).containsKey(lisConfig.getHisDetailName())) {
                        if (configMap.get(lisConfig.getHisName()).get(lisConfig.getHisDetailName()).containsKey(lisConfig.getUniqueName())) {
                            deleteIds.add(configMap
                                    .get(lisConfig.getHisName())
                                    .get(lisConfig.getHisDetailName())
                                    .get(lisConfig.getUniqueName()));
                        }
                    }
                }
                lisConfig.setCreator(userId);
                lisConfig.setGmtCreate(now);
            }
            if (lisConfig.getIsDeleted() == null) {
                lisConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);

        lisConfigService.saveOrUpdateBatch(lisConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<LisConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        lisConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<LisConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        lisConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param lisConfigPageVO
     * @return
     */
    public IPage<LisConfig> getPage(LisConfigPageVO lisConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        lisConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return lisConfigService.getPage(lisConfigPageVO);
    }

    /**
     * 化验数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        //从Excel读取数据
        List<LisConfig> lisConfigList = ExcelUtils.importExcel(file, 0, 1, LisConfig.class);
        if (ListUtil.isNotEmpty(lisConfigList)) {
            importExcelRecords(lisConfigList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param lisConfigList
     * @return
     */
    public Boolean importExcelRecords(List<LisConfig> lisConfigList) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        lisConfigList = lisConfigList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName())
                        || StringUtil.isNotBlank(i.getHisDetailName())
                        || StringUtil.isNotBlank(i.getUniqueCode())
                        || StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(lisConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < lisConfigList.size(); i++) {
            if (StringUtil.isBlank(lisConfigList.get(i).getHisName())
                    || StringUtil.isBlank(lisConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(lisConfigList.get(i).getHisName())) {
                lisConfigList.get(i).setHisName(lisConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(lisConfigList.get(i).getHisDetailName())) {
                lisConfigList.get(i).setHisDetailName(lisConfigList.get(i).getHisDetailName().trim());
            }
            if (StringUtil.isNotBlank(lisConfigList.get(i).getUniqueName())) {
                lisConfigList.get(i).setUniqueName(lisConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(lisConfigList.get(i).getUniqueCode())) {
                lisConfigList.get(i).setUniqueCode(lisConfigList.get(i).getUniqueCode().trim());
            } else {
                lisConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        List<String> hisNames = lisConfigList
                .stream()
                .map(i -> i.getHisName())
                .collect(Collectors.toList());
        List<String> uniqueNames = lisConfigList
                .stream()
                .map(i -> i.getUniqueName())
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Map<String, Long>>> configMap
                = getConfigMap(Long.valueOf(hospitalId), hisNames, uniqueNames);
        lisConfigList.forEach(lisConfig -> {
            lisConfig.setHospitalId(Long.valueOf(hospitalId));
            lisConfig.setModifier(userId);
            lisConfig.setGmtModified(now);
            if (lisConfig.getId() == null) {
                if (configMap.containsKey(lisConfig.getHisName())) {
                    if (lisConfig.getHisDetailName() == null
                            && configMap.get(lisConfig.getHisName()).containsKey("")) {
                        if (configMap.get(lisConfig.getHisName()).get("").containsKey(lisConfig.getUniqueName())) {
                            deleteIds.add(configMap.get(lisConfig.getHisName()).get("").get(lisConfig.getUniqueName()));
                        }
                    } else if (configMap.get(lisConfig.getHisName()).containsKey(lisConfig.getHisDetailName())) {
                        if (configMap.get(lisConfig.getHisName()).get(lisConfig.getHisDetailName()).containsKey(lisConfig.getUniqueName())) {
                            deleteIds.add(configMap
                                    .get(lisConfig.getHisName())
                                    .get(lisConfig.getHisDetailName())
                                    .get(lisConfig.getUniqueName()));
                        }
                    }
                }
                lisConfig.setCreator(userId);
                lisConfig.setGmtCreate(now);
            }
            if (lisConfig.getIsDeleted() == null) {
                lisConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });

        //标准术语校验
        List<String> errorNumList = Lists.newLinkedList();
        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setNames(uniqueNames);
        conceptVO.setType(ConceptTypeEnum.LisPack.getKey());
        RespDTO<List<String>> respLisPackDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respLisPackDTO, "标准术语校验失败");
        List<String> lisPackNames = respLisPackDTO.data;
        conceptVO.setType(ConceptTypeEnum.Lis.getKey());
        RespDTO<List<String>> respLisDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respLisDTO, "标准术语校验失败");
        List<String> lisNames = respLisDTO.data;
        for (int i = 0; i < lisConfigList.size(); i++) {
            if (!lisPackNames.contains(lisConfigList.get(i).getUniqueName())
                    && !lisNames.contains(lisConfigList.get(i).getUniqueName())) {
                errorNumList.add(String.valueOf(i + 2));
            }
        }
        if (ListUtil.isNotEmpty(errorNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "以下行数（不计入空行）标准术语与数据库术语不匹配："
                            + errorNumList.stream().collect(Collectors.joining("、"))
                            + "。导入取消，请修改后再试。");
        }

        //重复数据过滤
        lisConfigList = lisConfigList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        lisConfigService.saveOrUpdateBatch(lisConfigList);
        return true;
    }


    /**
     * 获取映射关系-id
     * Map<hisName,Map<hisDetailName,Map<uniqueName,id>>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Map<String, Long>>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Map<String, Long>>> retMap = new HashMap<>();
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<LisConfig> records = lisConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        records.forEach(lisConfig -> {
            if (lisConfig.getHisDetailName() == null) {
                lisConfig.setHisDetailName("");
            }
        });

        Map<String, List<LisConfig>> hisNameMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<LisConfig>> entry : hisNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, Map<String, Long>> detailNameMap = new HashMap<>();
                //his名称映射到标准词，1：n
                Map<String, List<LisConfig>> hisDetailNameMap
                        = EntityUtil.makeEntityListMap(entry.getValue(), "hisDetailName");
                for (Map.Entry<String, List<LisConfig>> detailEntry : hisDetailNameMap.entrySet()) {
                    if (ListUtil.isNotEmpty(detailEntry.getValue())) {
                        detailNameMap.put(detailEntry.getKey(),
                                EntityUtil.makeMapWithKeyValue(detailEntry.getValue(), "uniqueName", "id"));
                    }
                }
                retMap.put(entry.getKey(), detailNameMap);
            }
        }
        return retMap;
    }


    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<LisConfig> records = lisConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        records.forEach(lisConfig -> {
            if (lisConfig.getHisDetailName() == null) {
                lisConfig.setHisDetailName("");
            }
        });

        Map<String, List<LisConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<LisConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }


    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<LisConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID())
                .orderByDesc("gmt_modified");
        List<LisConfig> records = lisConfigService.list(queryWrapper);
        String fileName = "检验映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", LisConfig.class, fileName, response, 12.8f);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "检验映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", LisConfig.class, fileName, response, 12.8f);
    }

    /**
     * 化验大项
     *
     * @param retrievalVO
     * @return
     */
    public List<String> getLisPacks(RetrievalVO retrievalVO) {
        List<LisConfig> records = lisConfigService.getLisPacksIndex(retrievalVO);
        List<String> lisNames
                = records.stream().map(r -> r.getHisName()).distinct().collect(Collectors.toList());
        return lisNames;
    }

    /**
     * 化验小项
     *
     * @param retrievalVO
     * @return
     */
    public List<LisDetailDTO> getLis(RetrievalVO retrievalVO) {
        List<LisDetailDTO> lisDetailNames = new ArrayList<>();
        List<LisConfig> records = lisConfigService.getLisIndex(retrievalVO);
        for (LisConfig lisConfig : records) {
            LisDetailDTO lisDetailDTO = new LisDetailDTO();
            lisDetailDTO.setName(lisConfig.getHisName());
            lisDetailDTO.setUniqueName(lisConfig.getHisDetailName());
            lisDetailNames.add(lisDetailDTO);
        }
        return lisDetailNames;
    }
}