package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.MrtestInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.MrtestInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.GsonUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.MrtestInfoVO;
import com.diagbot.vo.SearchData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/10/26 12:11
 */
@Component
public class MrtestInfoFacade extends MrtestInfoServiceImpl {
    @Autowired
    @Qualifier("mrtestInfoServiceImpl")
    private MrtestInfoServiceImpl mrtestInfoService;
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private AssembleFacade assembleFacade;

    public void importExcel(MultipartFile file) {
        List<MrtestInfo> mrtestInfoList = ExcelUtils.importExcel(file, 0, 1, MrtestInfo.class);
        if (ListUtil.isNotEmpty(mrtestInfoList)) {
            mrtestInfoService.saveBatch(mrtestInfoList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response, MrtestInfoVO mrtestInfoVO) {
        List<MrtestInfo> mrtestInfoList
                = mrtestInfoService.list(
                new QueryWrapper<MrtestInfo>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq((null != mrtestInfoVO && null != mrtestInfoVO.getHospitalId()),
                                "hospital_id", mrtestInfoVO.getHospitalId())
                        .eq((null != mrtestInfoVO && null != mrtestInfoVO.getPass()),
                                "pass", mrtestInfoVO.getPass())
                        .in(ListUtil.isNotEmpty(mrtestInfoVO.getIds()), "id", mrtestInfoVO.getIds())
        );
        String fileName = "mrtest.xls";
        ExcelUtils.exportExcel(mrtestInfoList, null, "sheet1", MrtestInfo.class, fileName, response, 12.8f);
    }

    public Boolean mrTestProcess(MrtestInfoVO mrtestInfoVO) {
        List<MrtestInfo> mrtestInfoList
                = mrtestInfoService.list(
                new QueryWrapper<MrtestInfo>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq((null != mrtestInfoVO && null != mrtestInfoVO.getHospitalId()),
                                "hospital_id", mrtestInfoVO.getHospitalId())
                        .eq((null != mrtestInfoVO && null != mrtestInfoVO.getPass()),
                                "pass", mrtestInfoVO.getPass())
                        .in(ListUtil.isNotEmpty(mrtestInfoVO.getIds()), "id", mrtestInfoVO.getIds())
        );

        if (ListUtil.isEmpty(mrtestInfoList)) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS);
        } else {
            Date now = DateUtil.now();
            for (MrtestInfo mrtestInfo : mrtestInfoList) {
                //只处理目前存在的类型
                if (mrtestInfo.getType().equals(1)) {
                    if (mrtestInfo.getTextSource().equals("现病史")) {
                        if (StringUtil.isNotBlank(mrtestInfo.getReplaceText())) {
                            if (StringUtil.isNotBlank(mrtestInfo.getBillName())) {
                                IndicationPushVO indicationPushVO = new IndicationPushVO();
                                indicationPushVO.setHospitalId(mrtestInfo.getHospitalId());
                                indicationPushVO.setSex(1);
                                indicationPushVO.setAge("20");
                                indicationPushVO.setRuleType("2");
                                indicationPushVO.setSymptom(mrtestInfo.getReplaceText());
                                if ("医保手术和操作名称".equals(mrtestInfo.getBillType())) {
                                    List<Item> operationOrder = new ArrayList<>();
                                    Item item = new Item();
                                    item.setName(mrtestInfo.getBillName());
                                    operationOrder.add(item);
                                    indicationPushVO.setOperationOrder(operationOrder);
                                } else if ("辅助检查名称".equals(mrtestInfo.getBillType())
                                        || "辅助检查子项目名称".equals(mrtestInfo.getBillType())) {
                                    List<Pacs> pacsOrder = new ArrayList<>();
                                    Pacs pacs = new Pacs();
                                    pacs.setName(mrtestInfo.getBillName());
                                    pacsOrder.add(pacs);
                                    indicationPushVO.setPacsOrder(pacsOrder);
                                } else {
                                    mrtestInfo.setGmtModified(now);
                                    mrtestInfo.setResult("");
                                    mrtestInfo.setPass(0);
                                    mrtestInfo.setRemark("处理对象外数据");
                                    continue;
                                }

                                SearchData searchData = new SearchData();
                                BeanUtil.copyProperties(indicationPushVO, searchData);
                                //入参映射
                                searchData = assembleFacade.assembleData(searchData);
                                BeanUtil.copyProperties(searchData, indicationPushVO);
                                RespDTO<IndicationDTO> resp = cdssCoreClient.indication(indicationPushVO);
                                if (null != resp && null != resp.data) {
                                    mrtestInfo.setGmtModified(now);
                                    String billMsg = GsonUtil.toJson(resp.data.getBillMsgList());
                                    mrtestInfo.setResult(billMsg);
                                    if (StringUtil.isNotBlank(mrtestInfo.getReplaceWord())
                                            && billMsg.contains(mrtestInfo.getReplaceWord())) {
                                        mrtestInfo.setPass(1);
                                    } else {
                                        mrtestInfo.setPass(0);
                                    }

                                    mrtestInfo.setRemark("提醒接口有返回结果");
                                } else {
                                    mrtestInfo.setGmtModified(now);
                                    mrtestInfo.setResult("");
                                    mrtestInfo.setPass(0);
                                    mrtestInfo.setRemark("提醒接口无返回结果");
                                }
                            }
                        }
                    }
                } else {
                    mrtestInfo.setGmtModified(now);
                    mrtestInfo.setResult("");
                    mrtestInfo.setPass(0);
                    mrtestInfo.setRemark("处理对象外数据");
                }
            }
            mrtestInfoService.updateBatchById(mrtestInfoList);
        }
        return true;
    }
}
