package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.OperationConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.OperationConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.OperationConfigListVO;
import com.diagbot.vo.OperationConfigPageVO;
import com.diagbot.vo.RetrievalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:05
 */
@Component
public class OperationConfigFacade {
    @Autowired
    private OperationConfigService operationConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param operationConfig
     * @return
     */
    public Boolean isExistRecord(OperationConfig operationConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<OperationConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", operationConfig.getHisName())
                .eq("unique_name", operationConfig.getUniqueName());
        OperationConfig oldRecord = operationConfigService.getOne(queryWrapper, false);
        if (operationConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (operationConfig.getId() != null
                && oldRecord != null
                && !operationConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param operationConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(OperationConfig operationConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        operationConfig.setHospitalId(Long.valueOf(hospitalId));
        operationConfig.setModifier(userId);
        operationConfig.setGmtModified(now);
        QueryWrapper<OperationConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", operationConfig.getHisName())
                .eq("unique_name", operationConfig.getUniqueName());
        OperationConfig oldRecord = operationConfigService.getOne(queryWrapper, false);
        if (operationConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (operationConfig.getId() != null
                && oldRecord != null
                && !operationConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (operationConfig.getId() == null) {
            operationConfig.setCreator(userId);
            operationConfig.setGmtCreate(now);
        }
        if (operationConfig.getIsDeleted() == null) {
            operationConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        operationConfigService.saveOrUpdate(operationConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param operationConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(OperationConfigListVO operationConfigListVO) {
        if (ListUtil.isEmpty(operationConfigListVO.getOperationConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(operationConfigListVO.getOperationConfigList());
    }

    /**
     * 批量保存
     *
     * @param operationConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<OperationConfig> operationConfigList) {
        if (ListUtil.isEmpty(operationConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        operationConfigList = operationConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        operationConfigList.forEach(operationConfig -> {
            operationConfig.setHospitalId(Long.valueOf(hospitalId));
            operationConfig.setModifier(userId);
            operationConfig.setGmtModified(now);
            if (operationConfig.getId() == null) {
                if (configMap.containsKey(operationConfig.getHisName())) {
                    deleteIds.add(configMap.get(operationConfig.getHisName()).get(operationConfig.getUniqueName()));
                }
                operationConfig.setCreator(userId);
                operationConfig.setGmtCreate(now);
            }
            if (operationConfig.getIsDeleted() == null) {
                operationConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        operationConfigService.saveOrUpdateBatch(operationConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<OperationConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        operationConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<OperationConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        operationConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param operationConfigPageVO
     * @return
     */
    public IPage<OperationConfig> getPage(OperationConfigPageVO operationConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        operationConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return operationConfigService.getPage(operationConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<OperationConfig> operationConfigList = ExcelUtils.importExcel(file, 0, 1, OperationConfig.class);
        if (ListUtil.isNotEmpty(operationConfigList)) {
            importExcelRecords(operationConfigList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param operationConfigList
     * @return
     */
    public Boolean importExcelRecords(List<OperationConfig> operationConfigList) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        operationConfigList = operationConfigList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName())
                        || StringUtil.isNotBlank(i.getUniqueCode())
                        || StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(operationConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < operationConfigList.size(); i++) {
            if (StringUtil.isBlank(operationConfigList.get(i).getHisName())
                    || StringUtil.isBlank(operationConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(operationConfigList.get(i).getHisName())) {
                operationConfigList.get(i).setHisName(operationConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(operationConfigList.get(i).getUniqueName())) {
                operationConfigList.get(i).setUniqueName(operationConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(operationConfigList.get(i).getUniqueCode())) {
                operationConfigList.get(i).setUniqueCode(operationConfigList.get(i).getUniqueCode().trim());
            } else {
                operationConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        operationConfigList.forEach(operationConfig -> {
            operationConfig.setHospitalId(Long.valueOf(hospitalId));
            operationConfig.setModifier(userId);
            operationConfig.setGmtModified(now);
            if (operationConfig.getId() == null) {
                if (configMap.containsKey(operationConfig.getHisName())) {
                    deleteIds.add(configMap.get(operationConfig.getHisName()).get(operationConfig.getUniqueName()));
                }
                operationConfig.setCreator(userId);
                operationConfig.setGmtCreate(now);
            }
            if (operationConfig.getIsDeleted() == null) {
                operationConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });

        //标准术语校验
        List<String> errorNumList = Lists.newLinkedList();
        List<String> uniqueNames = operationConfigList.stream()
                .map(i -> i.getUniqueName())
                .distinct()
                .collect(Collectors.toList());
        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setNames(uniqueNames);
        conceptVO.setType(ConceptTypeEnum.Opeartion.getKey());
        RespDTO<List<String>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<String> names = respDTO.data;
        for (int i = 0; i < operationConfigList.size(); i++) {
            if (!names.contains(operationConfigList.get(i).getUniqueName())) {
                errorNumList.add(String.valueOf(i + 2));
            }
        }
        if (ListUtil.isNotEmpty(errorNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "以下行数（不计入空行）标准术语与数据库术语不匹配："
                            + errorNumList.stream().collect(Collectors.joining("、"))
                            + "。导入取消，请修改后再试。");
        }

        //重复数据过滤
        operationConfigList = operationConfigList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        operationConfigService.saveOrUpdateBatch(operationConfigList);
        return true;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<OperationConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<OperationConfig> records = operationConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<OperationConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<OperationConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<OperationConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<OperationConfig> records = operationConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        Map<String, List<OperationConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<OperationConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<OperationConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID())
                .orderByDesc("gmt_modified");
        List<OperationConfig> records = operationConfigService.list(queryWrapper);
        String fileName = "手术映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", OperationConfig.class, fileName, response, 12.8f);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "手术映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", OperationConfig.class, fileName, response, 12.8f);
    }

    /**
     * 手术搜索
     *
     * @param retrievalVO
     * @return
     */
    public List<OperationInfoDTO> getOperations(RetrievalVO retrievalVO) {
        List<OperationInfoDTO> operationNames = new ArrayList<>();
        List<OperationConfig> operationConfigList = operationConfigService.getOperationsIndex(retrievalVO);
        for (OperationConfig operationConfig: operationConfigList){
            OperationInfoDTO operationInfoDTO = new OperationInfoDTO();
            operationInfoDTO.setName(operationConfig.getHisName());
            operationNames.add(operationInfoDTO);
        }
        return operationNames;
    }
}