package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.PacsConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.PacsConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.PacsConfigListVO;
import com.diagbot.vo.PacsConfigPageVO;
import com.diagbot.vo.RetrievalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/7/29 15:03
 */
@Component
public class PacsConfigFacade {
    @Autowired
    private PacsConfigService pacsConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param pacsConfig
     * @return
     */
    public Boolean isExistRecord(PacsConfig pacsConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", pacsConfig.getHisName())
                .eq("unique_name", pacsConfig.getUniqueName());
        PacsConfig oldRecord = pacsConfigService.getOne(queryWrapper, false);
        if (pacsConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (pacsConfig.getId() != null
                && oldRecord != null
                && !pacsConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param pacsConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(PacsConfig pacsConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        pacsConfig.setHospitalId(Long.valueOf(hospitalId));
        pacsConfig.setModifier(userId);
        pacsConfig.setGmtModified(now);
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", pacsConfig.getHisName())
                .eq("unique_name", pacsConfig.getUniqueName());
        PacsConfig oldRecord = pacsConfigService.getOne(queryWrapper, false);
        if (pacsConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (pacsConfig.getId() != null
                && oldRecord != null
                && !pacsConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (pacsConfig.getId() == null) {
            pacsConfig.setCreator(userId);
            pacsConfig.setGmtCreate(now);
        }
        if (pacsConfig.getIsDeleted() == null) {
            pacsConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        pacsConfigService.saveOrUpdate(pacsConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param pacsConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(PacsConfigListVO pacsConfigListVO) {
        if (ListUtil.isEmpty(pacsConfigListVO.getPacsConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(pacsConfigListVO.getPacsConfigList());
    }

    /**
     * 批量保存
     *
     * @param pacsConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<PacsConfig> pacsConfigList) {
        if (ListUtil.isEmpty(pacsConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        pacsConfigList = pacsConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        pacsConfigList.forEach(pacsConfig -> {
            pacsConfig.setHospitalId(Long.valueOf(hospitalId));
            pacsConfig.setModifier(userId);
            pacsConfig.setGmtModified(now);
            if (pacsConfig.getId() == null) {
                if (configMap.containsKey(pacsConfig.getHisName())) {
                    deleteIds.add(configMap.get(pacsConfig.getHisName()).get(pacsConfig.getUniqueName()));
                }
                pacsConfig.setCreator(userId);
                pacsConfig.setGmtCreate(now);
            }
            if (pacsConfig.getIsDeleted() == null) {
                pacsConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        pacsConfigService.saveOrUpdateBatch(pacsConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<PacsConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        pacsConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<PacsConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        pacsConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param pacsConfigPageVO
     * @return
     */
    public IPage<PacsConfig> getPage(PacsConfigPageVO pacsConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        pacsConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return pacsConfigService.getPage(pacsConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<PacsConfig> pacsConfigList = ExcelUtils.importExcel(file, 0, 1, PacsConfig.class);
        if (ListUtil.isNotEmpty(pacsConfigList)) {
            importExcelRecords(pacsConfigList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param pacsConfigList
     * @return
     */
    public Boolean importExcelRecords(List<PacsConfig> pacsConfigList) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        pacsConfigList = pacsConfigList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName())
                        || StringUtil.isNotBlank(i.getUniqueCode())
                        || StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(pacsConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < pacsConfigList.size(); i++) {
            if (StringUtil.isBlank(pacsConfigList.get(i).getHisName())
                    || StringUtil.isBlank(pacsConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(pacsConfigList.get(i).getHisName())) {
                pacsConfigList.get(i).setHisName(pacsConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(pacsConfigList.get(i).getUniqueName())) {
                pacsConfigList.get(i).setUniqueName(pacsConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(pacsConfigList.get(i).getUniqueCode())) {
                pacsConfigList.get(i).setUniqueCode(pacsConfigList.get(i).getUniqueCode().trim());
            } else {
                pacsConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        pacsConfigList.forEach(pacsConfig -> {
            pacsConfig.setHospitalId(Long.valueOf(hospitalId));
            pacsConfig.setModifier(userId);
            pacsConfig.setGmtModified(now);
            if (pacsConfig.getId() == null) {
                if (configMap.containsKey(pacsConfig.getHisName())) {
                    deleteIds.add(configMap.get(pacsConfig.getHisName()).get(pacsConfig.getUniqueName()));
                }
                pacsConfig.setCreator(userId);
                pacsConfig.setGmtCreate(now);
            }
            if (pacsConfig.getIsDeleted() == null) {
                pacsConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });

        //标准术语校验
        List<String> errorNumList = Lists.newLinkedList();
        List<String> uniqueNames = pacsConfigList.stream()
                .map(i -> i.getUniqueName())
                .distinct()
                .collect(Collectors.toList());
        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setNames(uniqueNames);
        conceptVO.setType(ConceptTypeEnum.Pacs.getKey());
        RespDTO<List<String>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<String> names = respDTO.data;
        for (int i = 0; i < pacsConfigList.size(); i++) {
            if (!names.contains(pacsConfigList.get(i).getUniqueName())) {
                errorNumList.add(String.valueOf(i + 2));
            }
        }
        if (ListUtil.isNotEmpty(errorNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "以下行数（不计入空行）标准术语与数据库术语不匹配："
                            + errorNumList.stream().collect(Collectors.joining("、"))
                            + "。导入取消，请修改后再试。");
        }

        //重复数据过滤
        pacsConfigList = pacsConfigList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        pacsConfigService.saveOrUpdateBatch(pacsConfigList);
        return true;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<PacsConfig> records = pacsConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<PacsConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<PacsConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<PacsConfig> records = pacsConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        Map<String, List<PacsConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<PacsConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<PacsConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID())
                .orderByDesc("gmt_modified");
        List<PacsConfig> records = pacsConfigService.list(queryWrapper);
        String fileName = "检查映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", PacsConfig.class, fileName, response, 12.8f);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "检查映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", PacsConfig.class, fileName, response, 12.8f);
    }

    /**
     * 辅检
     *
     * @param retrievalVO
     * @return
     */
    public List<String> getPACSs(RetrievalVO retrievalVO) {
        List<PacsConfig> records = pacsConfigService.getPACSsIndex(retrievalVO);
        List<String> pacsNames
                = records.stream().map(r -> r.getHisName()).distinct().collect(Collectors.toList());
        return pacsNames;
    }
}