package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.PlanDetailDTO;
import com.diagbot.entity.PlanDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.PlanDetailService;
import com.diagbot.service.impl.PlanDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.HospitalPlanDetailSaveVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.PlanDetailCancelVO;
import com.diagbot.vo.PlanDetailRevStopVO;
import com.diagbot.vo.PlanDetailSaveVO;
import com.diagbot.vo.PlanDetailSubSaveVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-07-27 19:01
 */
@Component
public class PlanDetailFacade extends PlanDetailServiceImpl {

    @Autowired
    PlanDetailService planDetailService;


    public List<PlanDetailDTO> getSysSetInfoData(HospitalSetVO hospitalSetVO) {
        QueryWrapper<PlanDetail> sysSetInfo = new QueryWrapper<>();
        sysSetInfo.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalSetVO.getHospitalId())
                .eq(StringUtil.isNotBlank(hospitalSetVO.getCode()), "code", hospitalSetVO.getCode())
                .in("plan_id", hospitalSetVO.getPlanId())
                .orderByAsc("plan_id", "order_no");
        List<PlanDetail> sysSetData = list(sysSetInfo);
        List<PlanDetailDTO> sysSetInfoData = BeanUtil.listCopyTo(sysSetData, PlanDetailDTO.class);
        List<PlanDetailDTO> planDetailParent = new ArrayList<>();//父级数据
        List<PlanDetailDTO> planDetailSub = new ArrayList<>();//子级数据
        //分离数据，-1分父级菜单
        for (PlanDetailDTO dataDeta : sysSetInfoData) {
            if (dataDeta.getParentId() == (-1)) {
                planDetailParent.add(dataDeta);
            } else {
                planDetailSub.add(dataDeta);
            }
        }
        //把子级数据，放在父级下
        Map<Long, List<PlanDetailDTO>> mapParentId = planDetailSub.stream().collect(Collectors.groupingBy(PlanDetailDTO::getParentId));
        for (Long key : mapParentId.keySet()) {
            for (PlanDetailDTO sysPlanInfos : planDetailParent) {
                if (sysPlanInfos.getId().equals(key)) {
                    sysPlanInfos.setPlanDetails(mapParentId.get(key));
                }
            }
        }
        return planDetailParent;
    }

    public List<PlanDetailDTO> getByPlanIds(List<Long> planIds) {
        List<PlanDetailDTO> dataNew = new ArrayList<PlanDetailDTO>();
        if (ListUtil.isNotEmpty(planIds)) {
            QueryWrapper<PlanDetail> planDetailQuery = new QueryWrapper<>();
            planDetailQuery
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("STATUS", StatusEnum.Enable.getKey())
                    .in("plan_id", planIds)
                    .orderByDesc("gmt_create")
                    .orderByAsc("order_no");
            List<PlanDetail> datas = list(planDetailQuery);
            dataNew = BeanUtil.listCopyTo(datas, PlanDetailDTO.class);
        }
        return dataNew;
    }

    public boolean savePlanDetail(PlanDetailSaveVO planDetailSaveVO) {
        boolean res = false;
        Date now = DateUtil.now();
        QueryWrapper<PlanDetail> planDetailQuery = new QueryWrapper<>();
        planDetailQuery.eq("plan_id", planDetailSaveVO.getPlanId())
                .eq("hospital_id", planDetailSaveVO.getHospitalId())//医院id
                .eq("is_deleted", IsDeleteEnum.N.getKey());
        // 删除所有的配置
        res = remove(planDetailQuery);
        //先保存父类
        List<PlanDetail> planDetailParentNew = new ArrayList<>();
        List<HospitalPlanDetailSaveVO> planDetailParent = planDetailSaveVO.getPlanDetailParent();
        for (HospitalPlanDetailSaveVO planDetailSave : planDetailParent) {
            PlanDetail planDetails = new PlanDetail();
            BeanUtil.copyProperties(planDetailSave, planDetails);
            planDetails.setParentId(-1L);
            planDetails.setGmtCreate(now);
            planDetails.setCreator(SysUserUtils.getCurrentPrincipleID());
            planDetails.setModifier(SysUserUtils.getCurrentPrincipleID());
            planDetails.setGmtModified(now);
            planDetailService.save(planDetails);
            List<PlanDetailSubSaveVO> data = planDetailSave.getPlanDetailSub();
            if(ListUtil.isNotEmpty(data)){
                List<PlanDetail> planDetailList = new ArrayList<>();
                //再保存子类
                for (PlanDetailSubSaveVO planDetailVO : data) {
                    if(planDetailVO.getCode()!=null){
                        PlanDetail planDetail = new PlanDetail();
                        BeanUtil.copyProperties(planDetailVO, planDetail);
                        planDetail.setPlanId(planDetailSaveVO.getPlanId());
                        planDetail.setParentId(planDetails.getId());
                        planDetail.setGmtCreate(now);
                        planDetail.setCreator(SysUserUtils.getCurrentPrincipleID());
                        planDetail.setModifier(SysUserUtils.getCurrentPrincipleID());
                        planDetail.setGmtModified(now);
                        planDetailList.add(planDetail);
                    }
                }
                res = planDetailService.saveBatch(planDetailList);
            }
        }

        return res;
    }

    public boolean cancelPlanDetail(PlanDetailCancelVO planDetailCancelVO) {
        boolean res = false;
        // 1.先判断数据是否存在有效
        UpdateWrapper<PlanDetail> planDetailNew = new UpdateWrapper<>();
        planDetailNew
                .eq("hospital_id", planDetailCancelVO.getHospitalId())
                .eq("plan_id", planDetailCancelVO.getPlanId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("is_deleted", IsDeleteEnum.Y.getKey())
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        res = update(new PlanDetail(), planDetailNew);
        return res;
    }

    public boolean revStopPlanDetail(PlanDetailRevStopVO planDetailRevStopVO) {
        checkPlan(planDetailRevStopVO.getId());
        UpdateWrapper<PlanDetail> planNew = new UpdateWrapper<>();
        planNew
                .eq("id", planDetailRevStopVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("status", planDetailRevStopVO.getStatus())
                .set("modifier", UserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        return update(new PlanDetail(), planNew);

    }

    /**
     * 判断数据是否存在有效
     *
     * @param id
     */
    private PlanDetail checkPlan(Long id) {
        // 1.先判断数据是否存在有效
        QueryWrapper<PlanDetail> PlanFand = new QueryWrapper<>();
        PlanFand.eq("is_deleted", IsDeleteEnum.N.getKey()).eq("id", id);
        PlanDetail planDetail = getOne(PlanFand, false);
        if (null == planDetail) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该数据不存在");
        }
        return planDetail;
    }
}
