package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.TokenCheckDTO;
import com.diagbot.dto.TokenHospitaDTO;
import com.diagbot.entity.TokenHospital;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.TokenHospitalService;
import com.diagbot.service.impl.TokenHospitalServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RSATokenIdUtil;
import com.diagbot.vo.TokenHospitalVO;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-07-29 13:43
 */
@Component
public class TokenHospitalFacade extends TokenHospitalServiceImpl {

    @Autowired
    TokenHospitalService tokenHospitalService;

    /**
     * @param tokenHospitalVO
     * @return
     */
    public TokenHospitaDTO getTokenHospitals(TokenHospitalVO tokenHospitalVO) {
        TokenHospitaDTO tokenHospitaDTO = new TokenHospitaDTO();
        QueryWrapper<TokenHospital> tokenHospitalQuery = new QueryWrapper<>();
        tokenHospitalQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", tokenHospitalVO.getHospitalId());
        TokenHospital tokenHospital = getOne(tokenHospitalQuery, false);
        BeanUtil.copyProperties(tokenHospital, tokenHospitaDTO);
        return tokenHospitaDTO;
    }

    public TokenHospitaDTO getTokenInfo(String token) {
        TokenHospitaDTO tokenHospitaDTO = new TokenHospitaDTO();
        QueryWrapper<TokenHospital> tokenHospitalQuery = new QueryWrapper<>();
        tokenHospitalQuery.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("appid_token", token);
        TokenHospital tokenHospital = getOne(tokenHospitalQuery, false);
        BeanUtil.copyProperties(tokenHospital, tokenHospitaDTO);
        return tokenHospitaDTO;
    }

    /**
     * @param token
     * @return
     */
    public TokenCheckDTO checkTokenHospitals(String token) {
        Date now = DateUtil.now();
        boolean res = true;
        TokenHospitaDTO tokenHospitals = getTokenInfo(token);
        if(tokenHospitals!=null){
            int validDays = tokenHospitals.getValidDays();
            Date startTime = tokenHospitals.getStartTime();
            Date endTime = tokenHospitals.getEndTime();
            //校验数据是否人为修改了有效期
            StringBuffer sb = new StringBuffer();
            sb.append("Token")
                    .append(DateUtil.formatDateTime(startTime))// 开始时间
                    .append(DateUtil.formatDateTime(endTime)) //结束时间
                    .append(tokenHospitals.getHospitalId().toString()); // 医院id
            try {
                //解密Token
                String encrypt = RSATokenIdUtil.decrypt(token);
                if(!encrypt.equals(sb.toString())){
                    res = false;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (validDays <= 0) {
                res = false;
            }
            if (DateUtil.before(now, startTime)) {
                res = false;
            }
            if (DateUtil.after(now, endTime)) {
                res = false;
            }
        }
        TokenCheckDTO tokenCheckDTO = new TokenCheckDTO();
        tokenCheckDTO.setRes(res);
        tokenCheckDTO.setHospitalId(tokenHospitals.getHospitalId());
        tokenCheckDTO.setTokenId(tokenHospitals.getId());
        return tokenCheckDTO;
    }

    public Boolean refreshDays() {
        try {
            List<TokenHospital> tokenHospitals = this.list(new QueryWrapper<TokenHospital>()
                    .eq("is_deleted",  IsDeleteEnum.N.getKey())
                    .gt("valid_days",  0)
            );
            if(ListUtil.isNotEmpty(tokenHospitals)){
                for (TokenHospital tokenHospital : tokenHospitals) {
                    Integer validDays = tokenHospital.getValidDays();
                    if(validDays>0){
                        tokenHospital.setValidDays(validDays - 1);
                        tokenHospital.setGmtModified(DateUtil.now());
                    }
                }
                tokenHospitalService.updateBatchById(tokenHospitals);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "更新有效天数失败");
        }
        return true;
    }
}
