package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.TransfusionConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.TransfusionConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.TransfusionConfigListVO;
import com.diagbot.vo.TransfusionConfigPageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/31 11:03
 */
@Component
public class TransfusionConfigFacade {
    @Autowired
    private TransfusionConfigService transfusionConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param transfusionConfig
     * @return
     */
    public Boolean isExistRecord(TransfusionConfig transfusionConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<TransfusionConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", transfusionConfig.getHisName())
                .eq("unique_name", transfusionConfig.getUniqueName());
        TransfusionConfig oldRecord = transfusionConfigService.getOne(queryWrapper, false);
        if (transfusionConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (transfusionConfig.getId() != null
                && oldRecord != null
                && !transfusionConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param transfusionConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(TransfusionConfig transfusionConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        transfusionConfig.setHospitalId(Long.valueOf(hospitalId));
        transfusionConfig.setModifier(userId);
        transfusionConfig.setGmtModified(now);
        QueryWrapper<TransfusionConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", transfusionConfig.getHisName())
                .eq("unique_name", transfusionConfig.getUniqueName());
        TransfusionConfig oldRecord = transfusionConfigService.getOne(queryWrapper, false);
        if (transfusionConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        if (transfusionConfig.getId() != null
                && oldRecord != null
                && !transfusionConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (transfusionConfig.getId() == null) {
            transfusionConfig.setCreator(userId);
            transfusionConfig.setGmtCreate(now);
        }
        if (transfusionConfig.getIsDeleted() == null) {
            transfusionConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        transfusionConfigService.saveOrUpdate(transfusionConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param transfusionConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(TransfusionConfigListVO transfusionConfigListVO) {
        if (ListUtil.isEmpty(transfusionConfigListVO.getTransfusionConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(transfusionConfigListVO.getTransfusionConfigList());
    }

    /**
     * 批量保存
     *
     * @param transfusionConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<TransfusionConfig> transfusionConfigList) {
        if (ListUtil.isEmpty(transfusionConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        transfusionConfigList = transfusionConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        transfusionConfigList.forEach(transfusionConfig -> {
            transfusionConfig.setHospitalId(Long.valueOf(hospitalId));
            transfusionConfig.setModifier(userId);
            transfusionConfig.setGmtModified(now);
            if (transfusionConfig.getId() == null) {
                if (configMap.containsKey(transfusionConfig.getHisName())) {
                    deleteIds.add(configMap.get(transfusionConfig.getHisName()).get(transfusionConfig.getUniqueName()));
                }
                transfusionConfig.setCreator(userId);
                transfusionConfig.setGmtCreate(now);
            }
            if (transfusionConfig.getIsDeleted() == null) {
                transfusionConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        transfusionConfigService.saveOrUpdateBatch(transfusionConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<TransfusionConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        transfusionConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<TransfusionConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        transfusionConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param transfusionConfigPageVO
     * @return
     */
    public IPage<TransfusionConfig> getPage(TransfusionConfigPageVO transfusionConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        transfusionConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return transfusionConfigService.getPage(transfusionConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<TransfusionConfig> transfusionConfigList = ExcelUtils.importExcel(file, 0, 1, TransfusionConfig.class);
        if (ListUtil.isNotEmpty(transfusionConfigList)) {
            importExcelRecords(transfusionConfigList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param transfusionConfigList
     * @return
     */
    public Boolean importExcelRecords(List<TransfusionConfig> transfusionConfigList) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        transfusionConfigList = transfusionConfigList.stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName())
                        || StringUtil.isNotBlank(i.getUniqueCode())
                        || StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(transfusionConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < transfusionConfigList.size(); i++) {
            if (StringUtil.isBlank(transfusionConfigList.get(i).getHisName())
                    || StringUtil.isBlank(transfusionConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(transfusionConfigList.get(i).getHisName())) {
                transfusionConfigList.get(i).setHisName(transfusionConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(transfusionConfigList.get(i).getUniqueName())) {
                transfusionConfigList.get(i).setUniqueName(transfusionConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(transfusionConfigList.get(i).getUniqueCode())) {
                transfusionConfigList.get(i).setUniqueCode(transfusionConfigList.get(i).getUniqueCode().trim());
            } else {
                transfusionConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, Long>> configMap
                = getConfigMap(Long.valueOf(hospitalId), null, null);
        transfusionConfigList.forEach(transfusionConfig -> {
            transfusionConfig.setHospitalId(Long.valueOf(hospitalId));
            transfusionConfig.setModifier(userId);
            transfusionConfig.setGmtModified(now);
            if (transfusionConfig.getId() == null) {
                if (configMap.containsKey(transfusionConfig.getHisName())) {
                    deleteIds.add(configMap.get(transfusionConfig.getHisName()).get(transfusionConfig.getUniqueName()));
                }
                transfusionConfig.setCreator(userId);
                transfusionConfig.setGmtCreate(now);
            }
            if (transfusionConfig.getIsDeleted() == null) {
                transfusionConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });

        //标准术语校验
        List<String> errorNumList = Lists.newLinkedList();
        List<String> uniqueNames = transfusionConfigList.stream()
                .map(i -> i.getUniqueName())
                .distinct()
                .collect(Collectors.toList());
        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setNames(uniqueNames);
        conceptVO.setType(ConceptTypeEnum.Transfusion.getKey());
        RespDTO<List<String>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
        RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
        List<String> names = respDTO.data;
        for (int i = 0; i < transfusionConfigList.size(); i++) {
            if (!names.contains(transfusionConfigList.get(i).getUniqueName())) {
                errorNumList.add(String.valueOf(i + 2));
            }
        }
        if (ListUtil.isNotEmpty(errorNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "以下行数（不计入空行）标准术语与数据库术语不匹配："
                            + errorNumList.stream().collect(Collectors.joining("、"))
                            + "。导入取消，请修改后再试。");
        }

        //重复数据过滤
        transfusionConfigList = transfusionConfigList
                .stream()
                .distinct()
                .collect(Collectors.toList());

        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        transfusionConfigService.saveOrUpdateBatch(transfusionConfigList);
        return true;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<TransfusionConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<TransfusionConfig> records = transfusionConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<TransfusionConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<TransfusionConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<TransfusionConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<TransfusionConfig> records = transfusionConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        Map<String, List<TransfusionConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<TransfusionConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<TransfusionConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID())
                .orderByDesc("gmt_modified");
        List<TransfusionConfig> records = transfusionConfigService.list(queryWrapper);
        String fileName = "输血映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", TransfusionConfig.class, fileName, response, 12.8f);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "输血映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", TransfusionConfig.class, fileName, response, 12.8f);
    }

    /**
     * 输血搜索
     *
     * @param retrievalVO
     * @return
     */
    public List<String> getTransfusion(RetrievalVO retrievalVO) {
        List<TransfusionConfig> records = transfusionConfigService.getTransfusionIndex(retrievalVO);
        List<String> transfusionNames
                = records.stream().map(r -> r.getHisName()).distinct().collect(Collectors.toList());
        return transfusionNames;
    }
}