package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeHISDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.facade.ConceptInfoFacade;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.ConceptInfoPageVO;
import com.diagbot.vo.ConceptInfoVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.StaticKnowledgeHISVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.diagbot.vo.StaticKnowledgeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 静态知识标准术语表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2020-08-18
 */
@RestController
@RequestMapping("/graph/conceptInfo")
@Api(value = "静态知识标准术语相关API", tags = { "静态知识标准术语相关API" })
@SuppressWarnings("unchecked")
public class ConceptInfoController {

    @Autowired
    private ConceptInfoFacade conceptInfoFacade;

    @ApiOperation(value = "医学知识（静态知识）检索[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndex")
    @SysLogger("staticKnowledgeIndex")
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> data = conceptInfoFacade.staticKnowledgeIndexWithInfo(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医学术语检索-新增静态知识[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndexWithoutInfo")
    @SysLogger("staticKnowledgeIndexWithoutInfo")
    @Transactional
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndexWithoutInfo(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> data = conceptInfoFacade.staticKnowledgeIndexWithoutInfo(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "页面获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径、4-治疗方案<br>" +
                    "name： 标准术语名称<br>")
    @PostMapping("/getStaticKnowledge")
    @SysLogger("getStaticKnowledge")
    public RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO data = conceptInfoFacade.getStaticKnowledge(staticKnowledgeVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "对接获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径、4-治疗方案 <br>" +
                    "hisName： HIS大项名称<br>" +
                    "hisDetailName： HIS小项名称<br>")
    @PostMapping("/getStaticKnowledgeForHIS")
    @SysLogger("getStaticKnowledgeForHIS")
    public RespDTO<List<StaticKnowledgeHISDTO>> getStaticKnowledgeForHIS(@Valid @RequestBody StaticKnowledgeHISVO staticKnowledgeHISVO) {
        List<StaticKnowledgeHISDTO> data = conceptInfoFacade.getStaticKnowledgeForHIS(staticKnowledgeHISVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取静态知识列表[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作 <br>" +
                    "name： 术语名称<br>" +
                    "isDeleted： 启用状态：N-启用中、Y-已删除<br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<IPage<ConceptInfoDTO>> getPage(@Valid @RequestBody ConceptInfoPageVO conceptInfoPageVO) {
        IPage<ConceptInfoDTO> data = conceptInfoFacade.getPage(conceptInfoPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存静态知识-新增或修改[zhaops]",
            notes = "id： id <br>" +
                    "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作 <br>" +
                    "clinicalPathwayName： 临床路径名称<br>" +
                    "noticeName： 注意事项名称<br>" +
                    "details： 明细<br>")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody ConceptInfoVO conceptInfoVO) {
        Boolean data = conceptInfoFacade.saveOrUpdateRecord(conceptInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识启用禁用[zhaops]",
            notes = "id： 术语id <br>" +
                    "status： 启用状态（0：禁用，1：启用）<br>")
    @PostMapping("/changeStatus")
    @SysLogger("changeStatus")
    @Transactional
    public RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        Boolean data = conceptInfoFacade.changeStatus(changeStatusVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识是否存在[zhaops]",
            notes = "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作 <br>")
    @PostMapping("/isExist")
    @SysLogger("isExist")
    public RespDTO<Boolean> isExist(@Valid @RequestBody ConceptInfoVO conceptInfoVO) {
        Boolean data = conceptInfoFacade.isExist(conceptInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据术语id获取静态信息[zhaops]",
            notes = "id： 术语id <br>")
    @PostMapping("/getRecordById")
    @SysLogger("getRecordById")
    public RespDTO<ConceptInfoDTO> getRecordById(@Valid @RequestBody IdVO idVO) {
        ConceptInfoDTO data = conceptInfoFacade.getRecordById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "批量更新静态知识标志[zhaops]",
            notes = "")
    @PostMapping("/updateHasInfoStatusBatch")
    @SysLogger("updateHasInfoStatusBatch")
    public RespDTO<Boolean> updateHasInfoStatusBatch() {
        Boolean data = conceptInfoFacade.updateHasInfoStatusBatch();
        return RespDTO.onSuc(data);
    }
}