package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.HospitalInfoDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.facade.HospitalInfoFacade;
import com.diagbot.vo.HospitalCheckVO;
import com.diagbot.vo.HospitalInfoVO;
import com.diagbot.vo.HospitalSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 医院信息表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-07-29
 */
@RestController
@RequestMapping("/tran/hospitalInfo")
@Api(value = "医院信息API", tags = { "医院信息API" })
@SuppressWarnings("unchecked")
public class HospitalInfoController {

    @Autowired
    private HospitalInfoFacade hospitalInfoFacade;

    @ApiOperation(value = "医院信息——根据医院Id获取医院信息[by:wangfeng]",
            notes = "id: 医院id<br>")
    @PostMapping("/getHospitalInfoById")
    @SysLogger("getHospitalInfoById")
    public RespDTO<HospitalInfoDTO> getHospitalInfoById(@RequestBody @Valid HospitalInfoVO hospitalInfoVO) {
        HospitalInfoDTO data = hospitalInfoFacade.getHospitalInfoById(hospitalInfoVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医院信息——验证医院信息[by:wangfeng]",
            notes = "hospitalCode：医院编码<br>")
    @PostMapping("/checkHospital")
    @SysLogger("checkHospital")
    @ApiIgnore
    public RespDTO<Boolean> checkHospital(@RequestBody @Valid HospitalCheckVO hospitalCheckVO) {
        return RespDTO.onSuc(hospitalInfoFacade.checkHospital(hospitalCheckVO));
    }

    @ApiOperation(value = "医院信息——修改医院信息[by:zhaops]",
            notes = "id：医院id<br>" +
                    "connect：是否对接（0-不对接，1-对接）<br>")
    @PostMapping("/saveRecord")
    @SysLogger("saveRecord")
    @Transactional
    public RespDTO<Boolean> saveRecord(@RequestBody @Valid HospitalSaveVO hospitalSaveVO) {
        return RespDTO.onSuc(hospitalInfoFacade.saveRecord(hospitalSaveVO));
    }

    @ApiOperation(value = "医院信息——获取医院信息[by:zhaops]",
            notes = "")
    @PostMapping("/getHospitalInfo")
    @SysLogger("getHospitalInfo")
    public RespDTO<HospitalInfo> getHospitalInfo() {
        return RespDTO.onSuc(hospitalInfoFacade.getHospitalInfo());
    }

    @ApiOperation(value = "获取所有医院信息列表[by:wangfeng]", notes = "")
    @PostMapping("/getAllHospitalInfo")
    @SysLogger("getAllHospitalInfo")
    public RespDTO<List<HospitalInfoDTO>> getAllHospitalInfo() {
        List<HospitalInfoDTO> data = hospitalInfoFacade.getAllHospitalInfo();
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取所有启用的医院信息列表[by:gaodm]", notes = "")
    @PostMapping("/getAllEnableHospitalInfo")
    @SysLogger("getAllEnableHospitalInfo")
    public RespDTO<List<HospitalInfoDTO>> getAllEnableHospitalInfo() {
        List<HospitalInfoDTO> data = hospitalInfoFacade.getAllEnableHospitalInfo();
        return RespDTO.onSuc(data);
    }
}
