package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.LisConfig;
import com.diagbot.facade.LisConfigFacade;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.LisConfigListVO;
import com.diagbot.vo.LisConfigPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * 化验公表映射
 *
 * @author zhaops
 * @since 2020-07-28
 */
@RestController
@RequestMapping("/tran/lisConfig")
@Api(value = "化验公表映射API", tags = { "化验公表映射API" })
@SuppressWarnings("unchecked")
public class LisConfigController {
    @Autowired
    private LisConfigFacade lisConfigFacade;

    /**
     * 映射关系是否已存在
     *
     * @param lisConfig
     * @return
     */
    @ApiOperation(value = "映射关系是否已存在[by:zhaops]", notes = "")
    @PostMapping("/isExistRecord")
    @SysLogger("isExistRecord")
    public RespDTO<Boolean> isExistRecord(@RequestBody @Valid LisConfig lisConfig) {
        Boolean data = lisConfigFacade.isExistRecord(lisConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 保存或修改映射关系
     *
     * @param lisConfig
     * @return
     */
    @ApiOperation(value = "保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid LisConfig lisConfig) {
        Boolean data = lisConfigFacade.saveOrUpdateRecord(lisConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量保存或修改映射关系
     *
     * @param lisConfigListVO
     * @return
     */
    @ApiOperation(value = "批量保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecords")
    @SysLogger("saveOrUpdateRecords")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecords(@RequestBody @Valid LisConfigListVO lisConfigListVO) {
        Boolean data = lisConfigFacade.saveOrUpdateRecords(lisConfigListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 删除映射关系
     *
     * @param idVO
     * @return
     */
    @ApiOperation(value = "删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = lisConfigFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量删除映射关系
     *
     * @param idListVO
     * @return
     */
    @ApiOperation(value = "批量删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = lisConfigFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 分页查询
     *
     * @param lisConfigPageVO
     * @return
     */
    @ApiOperation(value = "分页查询[by:zhaops]", notes = "")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<LisConfig> getPage(@RequestBody @Valid LisConfigPageVO lisConfigPageVO) {
        IPage<LisConfig> data = lisConfigFacade.getPage(lisConfigPageVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 化验公表数据导入
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "化验公表数据导入[by:zhaops]",
            notes = "")
    @PostMapping("/importExcel")
    @SysLogger("importExcel")
    @Transactional
    public void importExcel(@RequestParam("file") MultipartFile file) {
        lisConfigFacade.importExcel(file);
    }

    /**
     * 数据导出
     *
     * @return
     */
    @ApiOperation(value = "数据导出[by:gaodm]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response) {
        lisConfigFacade.exportExcel(response);
    }

    /**
     * 数据导入模板导出
     *
     * @return
     */
    @ApiOperation(value = "数据导入模板导出[by:zhaops]",
            notes = "")
    @PostMapping("/exportExcelModule")
    @SysLogger("exportExcelModule")
    public void exportExcelModule(HttpServletResponse response) {
        lisConfigFacade.exportExcelModule(response);
    }
}