package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.annotation.TokenAuth;
import com.diagbot.dto.AnalyzeRunDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.MrqcFacade;
import com.diagbot.vo.AnalyzeCdsVO;
import com.diagbot.vo.AnalyzeRunVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Map;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/7/28 9:54
 */
@RestController
@RequestMapping("/sys/mrqc")
@Api(value = "病历质控API", tags = { "病历质控API" })
@SuppressWarnings("unchecked")
public class MrqcController {
    @Autowired
    private MrqcFacade mrqcFacade;

    @ApiOperation(value = "评分-运行质控[by:zhoutg]",
            notes = "    // 病历id\n" +
                    "    String behospitalCode;\n" +
                    "    // 医院ID\n" +
                    "    Long hospitalId;\n" +
                    "    // 模块id\n" +
                    "    Long modeId;\n" +
                    "    // 归档字段\n" +
                    "    String isPlacefile;")
    @PostMapping("/analyze_run")
    @SysLogger("analyze_run")
    @TokenAuth
    public RespDTO<AnalyzeRunDTO> analyzeRun(@Valid @RequestBody AnalyzeRunVO analyzeRunVO) {
        return RespDTO.onSuc(mrqcFacade.analyzeRun(analyzeRunVO));
    }

    @ApiOperation(value = "病历书写格式提醒[by:wangfeng]",
            notes = "")
    @PostMapping("/caseWritingPrompt")
    @SysLogger("caseWritingPrompt")
    @TokenAuth
    public RespDTO<Map<String, String>> caseWritingPrompt(@Valid @RequestBody AnalyzeCdsVO analyzeCdsVO) {
        return RespDTO.onSuc(mrqcFacade.caseWritingPrompt(analyzeCdsVO));
    }
}
