package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.MrtestInfoFacade;
import com.diagbot.vo.MrtestInfoVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 * 病历测试表 前端控制器
 * </p>
 *
 * @author gaodm
 * @since 2020-10-26
 */
@RestController
@RequestMapping("/demo/mrtestInfo")
@Api(value = "病历测试API", tags = { "病历测试API" })
@SuppressWarnings("unchecked")
public class MrtestInfoController {
    @Autowired
    private MrtestInfoFacade mrtestInfoFacade;

    /**
     * 数据导入
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "数据导入[by:gaodm]",
            notes = "")
    @PostMapping("/importExcel")
    @SysLogger("importExcel")
    @Transactional
    public void importExcel(@RequestParam("file") MultipartFile file) {
        mrtestInfoFacade.importExcel(file);
    }

    /**
     * 数据导出
     *
     * @return
     */
    @ApiOperation(value = "数据导出[by:gaodm]",
            notes = "hospitalId：医院id（可不填）;<br>" +
                    "ids： ID列表（可不填）<br>" +
                    "pass： 是否通过 0:未通过，1：未通过（可不填）<br>")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response, @RequestBody MrtestInfoVO mrtestInfoVO) {
        mrtestInfoFacade.exportExcel(response, mrtestInfoVO);
    }

    @ApiOperation(value = "执行病历测试[by:gaodm]",
            notes = "hospitalId：医院id（可不填）;<br>" +
                    "ids： ID列表（可不填）<br>" +
                    "pass： 是否通过 0:未通过，1：未通过（可不填）<br>")
    @PostMapping("/mrTestProcess")
    @SysLogger("mrTestProcess")
    @Transactional
    public RespDTO<Boolean> mrTestProcess(@RequestBody MrtestInfoVO mrtestInfoVO) {
        return RespDTO.onSuc(mrtestInfoFacade.mrTestProcess(mrtestInfoVO));
    }
}
