package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.annotation.TokenAuth;
import com.diagbot.dto.PlanDetailDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.PlanDetailFacade;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.PlanDetailCancelVO;
import com.diagbot.vo.PlanDetailRevStopVO;
import com.diagbot.vo.PlanDetailSaveVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 医院所有配置信息 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-07-27
 */
@RestController
@RequestMapping("/sys/planDetail")
@Api(value = "方案配置信息明细API", tags = { "方案配置信息明细API" })
@SuppressWarnings("unchecked")
public class PlanDetailController {
    @Autowired
    PlanDetailFacade planDetailFacade;

    @ApiOperation(value = "根据医院编码获取配置信息明细[by:wangfeng]", notes = " hospitalId ：医院id <必传>" +
            "List<Long> planId;方案id  code：明细code ")
    @PostMapping("/getPlanDetailDatas")
    @SysLogger("getPlanDetailDatas")
    public RespDTO<List<PlanDetailDTO>> getSysSetInfoDatas(@Valid @RequestBody HospitalSetVO hospitalSetVO) {
        List<PlanDetailDTO> data = planDetailFacade.getSysSetInfoData(hospitalSetVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存更新方案配置信息明细[by:wangfeng]", notes = "保存更新方案配置信息明细")
    @PostMapping("/savePlanDetails")
    @SysLogger("savePlanDetails")
    @Transactional
    public RespDTO<Boolean> savePlanDetails(@Valid @RequestBody PlanDetailSaveVO planDetailSaveVO) {
        boolean res = planDetailFacade.savePlanDetail(planDetailSaveVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "删除方案配置明细[by:wangfeng]", notes = "删除方案配置明细")
    @PostMapping("/cancelPlanDetails")
    @SysLogger("cancelPlanDetails")
    @Transactional
    @ApiIgnore
    public RespDTO<Boolean> cancelPlanDetails(@Valid @RequestBody PlanDetailCancelVO planDetailCancelVO) {
        boolean res = planDetailFacade.cancelPlanDetail(planDetailCancelVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "停用启用方案配置明细[by:wangfeng]", notes = "id:方案配置id<br>status:状态")
    @PostMapping("/revStopPlanDetails")
    @SysLogger("revStopPlanDetails")
    @Transactional
    @ApiIgnore
    public RespDTO<Boolean> revStopPlanDetails(@Valid @RequestBody PlanDetailRevStopVO planDetailRevStopVO) {
        boolean res = planDetailFacade.revStopPlanDetail(planDetailRevStopVO);
        return RespDTO.onSuc(res);
    }
}
