package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.annotation.TokenAuth;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.PushFacade;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.PushPlanVO;
import com.diagbot.vo.PushVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/6 9:49
 */
@RestController
@RequestMapping("/sys/push")
@Api(value = "推理相关API", tags = { "推理相关API" })
@SuppressWarnings("unchecked")
public class PushController {
    @Autowired
    private PushFacade pushFacade;

    @ApiOperation(value = "基础推理[by:zhaops]",
            notes = "featureType 类型(多选必填)，1：症状，4：查体结果，5：检验，6：检查，7：诊断，8：药品，9：手术，10：一般治疗<br>" +
                    "getStaticKnowledge 是否获取静态知识 0：不获取，1：获取")
    @PostMapping("/push")
    @SysLogger("push")
    @TokenAuth
    public RespDTO<PushDTO> push(@RequestBody @Valid PushVO pushVO) {
        pushVO.setGetStaticKnowledge(1);
        PushDTO data = pushFacade.push(pushVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "基础推理-接口模式[by:zhaops]",
            notes = "featureType 类型(多选必填)，1：症状，4：查体结果，5：检验，6：检查，7：诊断，8：药品，9：手术，10：一般治疗")
    @PostMapping("/pushApi")
    @SysLogger("pushApi")
    @TokenAuth
    public RespDTO<PushDTO> pushApi(@RequestBody @Valid PushVO pushVO) {
        PushDTO data = pushFacade.push(pushVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "开单合理项推理[by:zhaops]", notes = "ruleType（1:危急值提醒，2:开单合理项，3:高危药品、手术，4:其他提醒）")
    @PostMapping("/indicationPush")
    @SysLogger("indicationPush")
    @TokenAuth
    public RespDTO<IndicationDTO> indicationPush(@RequestBody @Valid IndicationPushVO indicationPushVO) {
        IndicationDTO data = pushFacade.indicationPush(indicationPushVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "推送持续检验检查计划API[zhaops]", notes = "operationName：随访手术")
    @PostMapping("/pushPlan")
    @TokenAuth
    public RespDTO<PushPlanDTO> pushPlan(@RequestBody PushPlanVO pushPlanVO) {
        PushPlanDTO pushPlanDTO = pushFacade.pushPlan(pushPlanVO);
        return RespDTO.onSuc(pushPlanDTO);
    }
}