package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TemplateInfoDTO;
import com.diagbot.dto.TemplateInfoPageDTO;
import com.diagbot.facade.TemplateInfoFacade;
import com.diagbot.vo.TemplateIdVO;
import com.diagbot.vo.TemplateInfoPageVO;
import com.diagbot.vo.TemplateInfoRevampVO;
import com.diagbot.vo.TemplateInfosIdVO;
import com.diagbot.vo.TemplateInfosVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @author wangfeng
 * @Description: 演示病历表 前端控制器
 * @date 2018年11月16日 上午9:14:51
 */
@RestController
@RequestMapping("/demo/templateInfo")
@Api(value = "演示病历API[by:wangfeng]", tags = { "演示病历API" })
@SuppressWarnings("unchecked")
public class TemplateInfoController {

    @Autowired
    private TemplateInfoFacade templateInfoFacade;

    @ApiOperation(value = "根据病历id修改病历名称[by:wangfeng]",
            notes = "id;//病历</br>modeName;"
            + "//病历名</br>doctorId;")
    @PostMapping("/updateByIdUsNames")
    @SysLogger("updateByIdUsNames")
    @Transactional
    public RespDTO<Boolean> updateByIdUsNames(@Valid @RequestBody TemplateInfoRevampVO templateInfoRevampVO) {
        boolean res = templateInfoFacade.updateByIdUsName(templateInfoRevampVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "保存病历病历[by:wangfeng]",
            notes = "modeName;//病历名字</br>" +
            "sex;//性别</br>" +
            "age;//年龄</br>" +
            "patName;//病人姓名</br>" +
            "doctorName;//医生姓名</br>" +
            "inquiryCode;//门诊号</br>" +
            "cardNo;//卡号</br>" +
            "deptName;//科室名称</br>" +
            "inquiryDate;//就诊时间</br>" +
            "preview;//预览文本</br>" +
            "dataJson;//病历数据</br>")
    @PostMapping("/saveTemplateInfo")
    @SysLogger("saveTemplateInfo")
    @Transactional
    public RespDTO<Boolean> saveTemplateInfo(@Valid @RequestBody TemplateInfosVO templateInfosVO) {
        boolean res = templateInfoFacade.saveTemplateInfo(templateInfosVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "根据病历id批量删除病历病历[by:wangfeng]",
            notes = "ids: 病历ids<br>'ids':'1,2,3,4,5,6'")
    @PostMapping("/cancelTemplateInfos")
    @SysLogger("cancelTemplateInfos")
    @Transactional
    public RespDTO<Boolean> cancelTemplateInfos(@Valid @RequestBody TemplateInfosIdVO templateInfosIdVO) {
        boolean res = templateInfoFacade.cancelTemplateInfo(templateInfosIdVO);
        return RespDTO.onSuc(res);
    }

    @ApiOperation(value = "分页查询病历病历列表[by:wangfeng]",
            notes = "sex：性别<br>"
                    + "name：病历名称")
    @PostMapping("/getTemplatePageAlls")
    @SysLogger("getTemplatePageAlls")
    public RespDTO<IPage<TemplateInfoPageDTO>> getTemplatePageAll(@Valid @RequestBody TemplateInfoPageVO templateInfoPageVO) {
        IPage<TemplateInfoPageDTO> data = templateInfoFacade.getTemplatePageAlls(templateInfoPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据病历id获取病历病历内容[by:wangfeng]",
            notes = "id: 请输入病历id")
    @PostMapping("/getTemplatebyId")
    @SysLogger("getTemplatebyId")
    public RespDTO<TemplateInfoDTO> getTemplatebyId(@Valid @RequestBody TemplateIdVO templateIdVO) {
        TemplateInfoDTO data = templateInfoFacade.getTemplatebyId(templateIdVO);
        return RespDTO.onSuc(data);
    }
}
