package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TokenHospitaDTO;
import com.diagbot.facade.TokenHospitalFacade;
import com.diagbot.vo.TokenHospitalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

/**
 * <p>
 * 用户-机构表映射表 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-07-29
 */
@RestController
@RequestMapping("/sys/tokenHospital")
@Api(value = "医院token信息API", tags = { "医院token信息API" })
@SuppressWarnings("unchecked")
public class TokenHospitalController {

    @Autowired
    private TokenHospitalFacade tokenHospitalFacade;

    @ApiOperation(value = "根据医院id获取医院token信息[by:wangfeng]",
            notes = "hospitalId：医院id<br>")
    @PostMapping("/getTokenHospital")
    @SysLogger("getTokenHospital")
    public RespDTO<TokenHospitaDTO> getTokenHospital(@RequestBody @Valid TokenHospitalVO tokenHospitalVO) {
        TokenHospitaDTO data = tokenHospitalFacade.getTokenHospitals(tokenHospitalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "刷新有限天数，将剩余天数减1[by:wangfeng]",
            notes = "")
    @PostMapping("/refreshDays")
    @SysLogger("refreshDays")
    @ApiIgnore
    @Transactional
    public RespDTO<Boolean> refreshDays() {
        Boolean res = tokenHospitalFacade.refreshDays();
        return RespDTO.onSuc(res);
    }

}
